""" versioning """
__title__ = 'emailtrail'
__description__ = 'Analyse hops taken by an Email to reach you. Get structured information about each hop - Hostnames, Protocol used, Timestamp, and Delay'
__version__ = '0.3.21'
__author__ = 'Akshay Kumar'
__author_email__ = 'akshay.kmr4321@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017 Akshay Kumar'
__url__ = 'https://github.com/akshayKMR/emailtrail'
