"""
EmailTrail
~~~~~~~~~~~~~~~~~~~~~
Analyse hops taken by an Email to reach you.
Get structured information about each hop - Hostnames, Protocol used, Timestamp, and Delay
"""
from .module import *     # noqa
# wildcard import is intentional
